#include <global.h>

#ifdef KBD_EXTENDED_FUNCTIONS
  /// scancode ostatnio wciśniętego przycisku
  uint8_t __kbd_last_key;

  ///status przycisku, kolejne bity oznaczają:
  ///0: stan, 0=zwolniony, 1=wciśnięty
  ///1: odczytany przy pomocy funkcji kbd_getLastStatus
  ///2: odczytany wewnątrz funkcji getChar
  ///
  ///każda funkcja korzystająca z dodatkowych bitów zmiennej OR'uje lub AND'uje ją
  ///oprócz funkcji readChar, która czyści wszystkie ekstra bity
  uint8_t __kbd_last_status;

/** Klawisze specjalne:
 * 0:CTRL, 1:ALT, 2:SHIFT
 */ 
  uint8_t __kbd_mod_keys;
#endif 

uint8_t kbd_write(uint8_t command, uint8_t param) {
    uint8_t i;

    do {
      if(ps2_write((uint8_t *)&KBD_DATA_PORT, KBD_DATA_PIN, (uint8_t *)&KBD_CLK_PORT, KBD_CLK_PIN, command) != 0) return 1;
      i = ps2_read((uint8_t *)&KBD_DATA_PORT, KBD_DATA_PIN, (uint8_t *)&KBD_CLK_PORT, KBD_CLK_PIN);
    } while(i==0 || i==0xfe);//i powinno być 0xfa = ack

    if(param!=0xff) {
      do {
	if(ps2_write((uint8_t *)&KBD_DATA_PORT, KBD_DATA_PIN, (uint8_t *)&KBD_CLK_PORT, KBD_CLK_PIN, param) != 0) return 2;
	i = ps2_read((uint8_t *)&KBD_DATA_PORT, KBD_DATA_PIN, (uint8_t *)&KBD_CLK_PORT, KBD_CLK_PIN);
      } while(i==0 || i==0xfe);//j/w
    }

return 0;
}

uint8_t kbd_read() {
  return ps2_read((uint8_t *)&KBD_DATA_PORT, KBD_DATA_PIN, (uint8_t *)&KBD_CLK_PORT, KBD_CLK_PIN);
}

uint8_t kbd_readChar() {
  uint8_t tmp;
  tmp = kbd_read();
  if(tmp == 0xe0) tmp = kbd_read();
  if(tmp == 0xf0) {
    tmp = kbd_read();
    #ifdef KBD_EXTENDED_FUNCTIONS
      __kbd_last_status = 0;
      kbd_checkModKey(tmp, 0);
    #endif
    return 0;
  } else {
    #ifdef KBD_EXTENDED_FUNCTIONS
      __kbd_last_status = 1;
      __kbd_last_key = tmp;
      kbd_checkModKey(tmp, 1);
    #endif
    return tmp;
  } 
}

uint8_t kbd_setDiodes(uint8_t diodes) {
  return kbd_write(KBD_CMD_DIODES, diodes);
}


#ifdef KBD_EXTENDED_FUNCTIONS

void kbd_checkModKey(uint8_t code, uint8_t status) {
  if(status == 0) {//klawisz zwolniony
    if(code == 0x14)
      __kbd_mod_keys &= ~_BV(0);
    else if(code == 0x11)
      __kbd_mod_keys &= ~_BV(1);
    else if(code == 0x12)
      __kbd_mod_keys &= ~_BV(2);
  } else if(status == 1) {//klawisz wciśnięty
    if(code == 0x14)
      __kbd_mod_keys |= _BV(0);
    else if(code == 0x11)
      __kbd_mod_keys |= _BV(1);
    else if(code == 0x12)
      __kbd_mod_keys |= _BV(2);
  }
}

uint8_t kbd_getLastKey() {
  return __kbd_last_key;
}

uint8_t kbd_getLastStatus() {
  uint8_t tmp = __kbd_last_status;
  /*if(bit_is_set(__kbd_last_status,0)) */__kbd_last_status |= 2;
  return tmp;
}

uint8_t kbd_getModKeys() {
  return __kbd_mod_keys;
}

#ifdef KBD_ENABLE_ASCII_CONVERTER
uint8_t kbd_getChar() {
  if(bit_is_set(__kbd_last_status, 0) && bit_is_clear(__kbd_last_status,2)) {
    __kbd_last_status |= 4;
    return kbd_toASCII(__kbd_last_key);
  } else 
  	return 0;
}
#endif /* KBD_ENABLE_ASCII_CONVERTER */
#endif

#ifdef KBD_ENABLE_ASCII_CONVERTER
uint8_t kbd_toASCII(uint8_t code) {
  switch(code) {
    case 0x1c: return 'a';
    case 0x32: return 'b';
    case 0x21: return 'c';
    case 0x23: return 'd';
    case 0x24: return 'e';
    case 0x2b: return 'f';
    case 0x34: return 'g';
    case 0x33: return 'h';
    case 0x43: return 'i';
    case 0x3b: return 'j';
    case 0x42: return 'k';
    case 0x4b: return 'l';
    case 0x3a: return 'm';
    case 0x31: return 'n';
    case 0x44: return 'o';
    case 0x4d: return 'p';
    case 0x15: return 'q';
    case 0x2d: return 'r';
    case 0x1b: return 's';
    case 0x2c: return 't';
    case 0x3c: return 'u';
    case 0x2a: return 'v';
    case 0x1d: return 'w';
    case 0x22: return 'x';
    case 0x35: return 'y';
    case 0x1a: return 'z';
    case 0x45: return '0';
    case 0x16: return '1';
    case 0x1e: return '2';
    case 0x26: return '3';
    case 0x25: return '4';
    case 0x2e: return '5';
    case 0x36: return '6';
    case 0x3d: return '7';
    case 0x3e: return '8';
    case 0x46: return '9';
    case 0x29: return ' ';

    default:
      return 0;
  }

}
#endif

