#ifndef __KEYBOARD_H
#define __KEYBOARD_H

/**
 * Obsługa standardowej klawiatury PS/2
 * wersja 1
 * 
 * slis@edap.pl
 * http://edap.pl/
 */

/*
#define KBD_DATA_PORT	PORTB
#define KBD_DATA_PIN	PB3
#define KBD_CLK_PORT	PORTB
#define KBD_CLK_PIN	PB2
*/

/** Stała określająca, czy w programie mają być dostępne dodatkowe funkcje:
  * - zapamiętanie kodu ostatnio wciśniętego klawisza
  * - zapamiętanie stanu ostatniego klawisza (czy jest wciśnięty czy zwolniony)
  */
// #define KBD_EXTENDED_FUNCTIONS 1

/** Określa, czy w programie ma być dostępna funkcja konwertująca scancode do ASCII */
// #define KBD_ENABLE_ASCII_CONVERTER 1



#define KBD_DIODE_SCROLL	1
#define KBD_DIODE_CAPS		4
#define KBD_DIODE_NUM		2

#define KBD_CMD_DIODES		 0xED
#define KBD_CMD_RESET		 0xFF
#define KBD_CMD_REND		 0xFE
#define KBD_CMD_RATE		 0xF3
#define KBD_CMD_DISABLE		 0xF5
#define KBD_CMD_ENABLE		 0xF4
#define KBD_CMD_DEVICEID	 0xF2
#define KBD_CMD_CODESET		 0xF0
#define KBD_CMD_ECHO		 0xEE

#define KBD_KEY_ARROWDOWN   0x72
#define KBD_KEY_ARROWUP     0x75
#define KBD_KEY_ARROWLEFT   0x6B
#define KBD_KEY_ARROWRIGHT  0x74
#define KBD_KEY_ENTER       0x5A
#define KBD_KEY_ESC         0x76

#define KBD_KEY_F1          0x05


/**
 * Wysyła polecenie do klawiatury.
 * @param command Polecenie do wysłania
 * @param param parametr polecenia. Jeśli param==0xff polecenie nie jest wysyłane
 * @return 0, jeśli ok, 1 lub 2 jeśli wystąpił błąd podczas transmisji, lub kod zwrócony przez klawiaturę
 */
uint8_t kbd_write(uint8_t command, uint8_t param);

/**
 * Odczytuje bajt z klawiatury 
 * @return Odczytany bajt
 */
uint8_t kbd_read();

/**
 * Odczytuje kod wciśniętego klawisza i zwraca go. Jeśli żaden klawisz nie został wciśnięty, lub 
 * jakiś został puszczony, zwraca 0. Kod jest zwracany jako scancode klawiatury, tak więc musi 
 * zostać dodatkowo "przetłumaczony" na ASCII
 * W przypadku wykrycia kodu F0 (klawisz puszczony) lub klawisza rozszerzonego (E0) funckja czyta
 * dodatkowe bajty
 * @return Kod klawisza lub 0
 */
uint8_t kbd_readChar();

/**
 * Ustawia diody na klawiaturze
 * @param diodes Ustawienie diod, kombinacja KBD_DIODE_*, lub 0 aby wyłączyć wszystkie
 * @return zwrócony kod błędu, lub 0 jeśli ok
 */
uint8_t kbd_setDiodes(uint8_t diodes);

/// Ważne! większość funkcji rozszerzonych wymaga wcześniejszego wywołania funkcji kbd_readChar
#ifdef KBD_EXTENDED_FUNCTIONS
  /**
   * Zwraca kod ostatnio wciśniętego klawisza. Nie powoduje odczytania kodu z klawiatury, jedynie 
   * zwraca ostatnio zapamiętany kod
   * @return Kod klawisza (scancode) lub 0
   */
  uint8_t kbd_getLastKey();

  #ifdef KBD_ENABLE_ASCII_CONVERTER
    /**
    * Funkcja działa podobnie do getLastKey, z dwiema różnicami: 
    * 1. zwraca kod klawisza w jego postaci w ASCII
    * 2. zwraca kod klawisza tylko, jeśli został wciśnięty
    * Kod klawisza zostanie zwrócony tylko raz dla każdego odczytu, tzn dla drugiego odczytu zostanie zwrócone
    * 0 nawet, jeśli klawisz nie został zwolniony
    * @return Kod wciśniętego klawisza w ASCII, lub 0
    */
    uint8_t kbd_getChar();
  #endif

  /**
   * Zwraca stan ostatniego klawisza (wciśnięty lub puszczony). Podobnie, jak getLastKey nie powoduje
   * odczytania kodu
   * @return 0 - klawisz zwolniony, 1 - klawisz wciśnięty, 3 - klawisz wciśnięty, ale był już sprawdzany
   * tą funkcją
   */
  uint8_t kbd_getLastStatus();

  /**
   * Zwraca stan przycisków specjalnych
   * @return 0:CTRL, 1:ALT, 2:SHIFT
   */
  uint8_t kbd_getModKeys();


  /**
   * Funkcja wewnętrzna, do zapamiętania stanu klawiszy specjalnych (alt,shift,ctrl)
   */
  void kbd_checkModKey(uint8_t code, uint8_t status);
#endif

#ifdef KBD_ENABLE_ASCII_CONVERTER
  /**
   * Konwertuje scancode klawiatury do postaci ASCII. Klawisze sterujące (np strzałki) nie są konwertowane (zwraca 0)
   * @param code scancode klawiatury
   * @return kod klawisza w ascii
   */
  uint8_t kbd_toASCII(uint8_t code);
#endif


#endif
